/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.aokell.tools.content;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;

import org.objectweb.fractal.aokell.lib.membrane.marker.LifeCycleInterceptorType;


/**
 * This class contains helper methods with an implementation which differs
 * depending on the choosen version of the glue feature. These methods are used
 * by the content resolver tool.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class ContentResolverHelper {
    
    private PrintWriter pw;
    
    public void newFile(
            String generatedDirname, String packagename, String classname )
        throws IOException
    {        
        String dirname = generatedDirname+"/"+packagename.replace('.','/');
        String aspectname = "A"+classname;
        String filename = dirname+"/"+aspectname+".aj";
        new File(dirname).mkdirs();

        System.out.println("Writing into: "+filename);
        
        FileWriter fw = new FileWriter(filename);
        pw = new PrintWriter(fw);
        
        pw.println("/*");
        pw.print(" * Generated by AOKell ContentResolver on: ");
        pw.println(new Date().toString());
        pw.println(" */");
        pw.println();
        
        pw.println("package "+packagename+";");
        pw.println();
        pw.println("public aspect "+aspectname+" {");
    }

    public void generateClause( Object contentDesc, Object ctrlDesc )
    throws IOException {
        
        if( ((String)ctrlDesc).endsWith("Template") ) {
            return;
        }
        
        pw.print("  declare parents: ");
        String msg = contentDesc+" implements "+LifeCycleInterceptorType.class.getName();
        pw.print(msg);
        pw.print(";  // ");
        pw.println(ctrlDesc);
        
        System.out.println("  "+msg);
    }

    public void epilogue() throws IOException {
        pw.println("}");
        pw.close();
    }
}
