/***
 * AOKell
 * Copyright (C) 2005-2006 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package org.objectweb.fractal.aokell.lib.interf;

/**
 * This class contains utility methods related to the management of classes
 * implementing Fractal interfaces.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class FcItfImplHelper {

    /**
     * Compute the name of the package which will hold the class implementing
     * the Fractal interface whose signature is given as a parameter.
     */
    public static String getGenPkgName( String pkgName ) {        
        return
            (pkgName.length()==0) ? 
            genByAOKellPkgName : genByAOKellPkgName + "." + pkgName;
    }
    
    /** The package name extension for classes generated by AOKell. */
    final private static String genByAOKellPkgName = "aokell.generated";

    
    /**
     * Return the Java Bean compliant field name associated to a given setter
     * method name. E.g. getFieldNameFromSetter("setFooBar") returns "fooBar".
     */
    public static String getFieldNameFromSetter( String setter ) {
        
        if( setter.length() < 4 || ! setter.startsWith("set") ) {
            final String msg =
                setter +
                " does not follow Java Bean conventions for a setter name";
            throw new IllegalArgumentException(msg);
        }
        
        return setter.substring(3,4).toLowerCase() + setter.substring(4);
    }
}
