/***
 * Fractal Hello World Example
 * Copyright (C) 2001-2002 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Eric Bruneton
 * Author: Lionel Seinturier
 */

package cs.impl;

import org.objectweb.dream.control.logger.Loggable;
import org.objectweb.fractal.aokell.dream.lib.control.logger.LoggerControllerItf;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


public class ClientImpl implements Runnable, Loggable {

  public ClientImpl () {
    System.err.println("CLIENT created");
  }
  
  public void run () {
      
      Component comp = (Component)this;
      
      System.out.println();
      System.out.println("List of interfaces implemented by CLIENT");
      Object[] itfs = comp.getFcInterfaces();
      for (int i = 0; i < itfs.length; i++) {
        Interface itf = (Interface) itfs[i];
        System.out.println(i+" "+itf.getFcItfName());
      }
      System.out.println();
      
      try {
          LoggerControllerItf lci = (LoggerControllerItf) comp.getFcInterface("logger-controller");
          lci.register("myLogger",this);
      }
      catch( NoSuchInterfaceException nsie ) {
          System.err.println( nsie.getMessage() );
      }
      
      Service service = null;
      try {
          logger.log(BasicLevel.INFO,"Component s called");
          service = (Service) ((BindingController)this).lookupFc("s");
      }
      catch( NoSuchInterfaceException nsie ) {
          throw new RuntimeException(nsie.getMessage());
      }
      
      service.print("hello world");
  }

  private Logger logger;
  
  public void setLogger(String name, Logger logger) {
      this.logger = logger;
//      this.logger.setLevel(BasicLevel.LEVEL_FATAL);
  }

}
